/* See accompanying file, LICENSE, for license information */

if(typeof BUBBAMON === 'undefined') {
    var BUBBAMON = {};
}

BUBBAMON.prefs = null;
BUBBAMON.sessionCookie = "";
BUBBAMON.serverType = 0;
BUBBAMON.server = null;
BUBBAMON.detecting = false;     // Only one detection thread.
BUBBAMON.locale = null;

BUBBAMON.LOG = function(msg) {
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage(msg);
}

BUBBAMON.dumpObj = function (obj, name, indent, depth) {
    if (depth > 10) {
        return indent + name + ": <Maximum Depth Reached>\n";
    }
    if (typeof obj == "object") {
        var child = null;
        var output = indent + name + "\n";
        indent += "\t";
        for (var item in obj)
        {
            try {
                child = obj[item];
            } catch (e) {
                child = "<Unable to Evaluate>";
            }
            if (typeof child == "object") {
                output += BUBBAMON.dumpObj(child, item, indent, depth + 1);
            } else {
                output += indent + item + ": " + child + "\n";
            }
        }
        return output;
    } else {
        return obj;
    }
}

BUBBAMON.getPassword = function(username) {
    var pwdManager = new BUBBAMON.PasswordManager(username);
    return pwdManager.get();
}

/** @brief Preferences class.
 *
 * All preferences are stored and handled by this class.
 */
BUBBAMON.Preferences = function()  {
    // Initialize preferences
    this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService)
        .getBranch("extensions.bubbamon.");
    this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this.prefs.addObserver("", this, false);

    this.address = this.prefs.getCharPref("address");
    this.user = this.prefs.getCharPref("user");
    this.interval = this.prefs.getIntPref("interval") * 1000;
    this.overview = this.prefs.getIntPref("overview_interval") * 1000;
    this.passwd = BUBBAMON.getPassword(this.user);
    this.hddtemp = this.prefs.getCharPref("hddtemp");
    this.intermediateDst = this.prefs.getCharPref("intermediateDst");
    this.intermediateSrc = this.prefs.getCharPref("intermediateSrc");
    this.autoDelete = this.prefs.getBoolPref("autodelete");
    this.downloadMode = this.prefs.getIntPref("downloadmode");
    this.serverType = this.prefs.getIntPref("serverType");
    this.nagCounter = 0;
    this.ongoing = {
        'left': this.prefs.getCharPref("ongoing_left"),
        'upper': this.prefs.getCharPref("ongoing_upper"),
        'lower': this.prefs.getCharPref("ongoing_lower")
    }
    this.completed = {
        'left': this.prefs.getCharPref("completed_left"),
        'upper': this.prefs.getCharPref("completed_upper"),
        'lower': this.prefs.getCharPref("completed_lower")
    }


    this.destroy = function() {
        this.prefs.removeObserver("", this);
    };

    this.observe = function(subject, topic, data) {
        if (topic != "nsPref:changed") {
            return;
        }

        switch(data)
        {
            case "address":
                this.address = this.prefs.getCharPref("address");
                this.serverType = 0;
                break;
            case "user":
                this.nagCounter = 0;
                this.user = this.prefs.getCharPref("user");
                this.passwd = BUBBAMON.getPassword(this.user);
                this.serverType = 0;
                break;
            case "password":
                this.nagCounter = 0;
                this.passwd = BUBBAMON.getPassword(this.user);
                this.serverType = 0;
                break;
            case "interval":
                this.interval = this.prefs.getIntPref("interval") * 1000;
                break;
            case "overview_interval":
                this.overview = this.prefs.getIntPref("overview_interval") * 1000;
                break;
            case "hddtemp":
                this.hddtemp = this.prefs.getCharPref("hddtemp");
                break;
            case "intermediateDst":
                this.intermediateDst = this.prefs.getCharPref("intermediateDst");
                break;
            case "intermediateSrc":
                this.intermediateSrc = this.prefs.getCharPref("intermediateSrc");
                break;
            case "autodelete":
                this.autoDelete = this.prefs.getBoolPref("autodelete");
                break;
            case "downloadmode":
                this.downloadMode = this.prefs.getIntPref("downloadmode");
                break;
            case "serverType":
                this.serverType = this.prefs.getIntPref("serverType");
                break;
            case "ongoing_left":
                this.ongoing['left'] = this.prefs.getCharPref("ongoing_left");
                break;
            case "ongoing_upper":
                this.ongoing['upper'] = this.prefs.getCharPref("ongoing_upper");
                break;
            case "ongoing_lower":
                this.ongoing['lower'] = this.prefs.getCharPref("ongoing_lower");
                break;
            case "completed_left":
                this.completed['left'] = this.prefs.getCharPref("completed_left");
                break;
            case "completed_upper":
                this.completed['upper'] = this.prefs.getCharPref("completed_upper");
                break;
            case "completed_lower":
                this.completed['lower'] = this.prefs.getCharPref("completed_lower");
                break;
            default:
                alert("Unknown preference tag: " + data);
        }
        if(this.serverType == 0) {
            if(BUBBAMON.server)
            {
                BUBBAMON.server.logOff();
            }
            if((this.address != "" || this.user != "" || this.passwd != null)) {
                this.configured = true;
                BUBBAMON.detectServer();
            }
        }
    }

    // Test that preferences are set.
    if(this.address == "" || this.user == "" || this.passwd == null) {
        BUBBAMON.setStatus(BUBBAMON.locale.getText('bubbamon.hint.not_configured'));
        this.configured = false;
    } else {
        this.configured = true;
    }

}

/** @brief Build a XUL progress bar.
 *
 * This will build a progress bar inside a stack element.
 * It is used to build the progress bars shown in downloads and disk 
 * information.
 *
 * @param title Title shown to the left
 * @param progress Progress, 0-100, -1 = undefined
 * @param upper Upper information, shown in top right corner.
 * @param lower Lower information, shown in lower right corner.
 */
BUBBAMON.ProgressBar = function(title, progress, upper, lower) {
    this.title = title;
    this.upper = upper;
    this.lower = lower;
    this.progress = progress;
    this.id = title;

    this.class = "BM_BAR_progress";

    this.asXUL = function(textFields) {
        var stack = document.createElement("stack");
        stack.setAttribute("flex", "1");
        stack.setAttribute("id", this.id);
        stack.setAttribute("class", "BM_progress");

        var bar = document.createElement("hbox");
        bar.setAttribute("class", this.class);
        stack.appendChild(bar);

        var tmp = document.createElement("hbox");
        tmp.setAttribute("flex", "1");
        switch(this.progress) {
            case -1:
                tmp.setAttribute("class", "BM_BAR_unknown");
                bar.appendChild(tmp);
                break;
            case -2:
                tmp.setAttribute("class", "BM_BAR_error");
                bar.appendChild(tmp);
                break;
            case -3:
                tmp.setAttribute("class", "BM_BAR_none");
                bar.appendChild(tmp);
                break;
            case 100:
                tmp.setAttribute("class", "BM_BAR_done");
                tmp.setAttribute("flex", "1");
                bar.appendChild(tmp);
                break;
            default:
                tmp.setAttribute("class", "BM_BAR_done");
                tmp.setAttribute("flex", this.progress);
                bar.appendChild(tmp);
                tmp = document.createElement("hbox");
                tmp.setAttribute("class", "BM_BAR_remaining");
                tmp.setAttribute("flex", (100-this.progress) + '');
                bar.appendChild(tmp);
                break;
        }

        bar = document.createElement("hbox");
        bar.setAttribute("align", "center");
        stack.appendChild(bar);

        // Title, shown to the left
        tmp = document.createElement("label");
        tmp.setAttribute("class", "BM_BAR_left");
        tmp.setAttribute("value", this.title);
        if(null != textFields) {
            tmp.setAttribute("tooltiptext", textFields['name'] + "\nSize: " + textFields['size'] + "\nDown: " + textFields['down'] + " (" + textFields['downloaded'] + ")\nUp: " + textFields['up'] + " (" + textFields['uploaded'] + ")");
        }
        tmp.setAttribute("crop", "end");
        tmp.setAttribute("flex", "1");
        bar.appendChild(tmp);

        // Additional info, show to the left
        var vbox = document.createElement("vbox");
        vbox.setAttribute("class", "bubba_progressInfo");
        vbox.setAttribute("pack", "center");

        // Upper part
        tmp = document.createElement("label");
        tmp.setAttribute("class", "BM_BAR_right");
        tmp.setAttribute("value", this.upper);
        vbox.appendChild(tmp);

        // Lower part
        tmp = document.createElement("label");
        tmp.setAttribute("class", "BM_BAR_right");
        tmp.setAttribute("value", this.lower);
        vbox.appendChild(tmp);

        bar.appendChild(vbox);

        return stack;
    }
}

/** @brief Update text field next to icon with a new text.
 *
 * @param msg Text message that will appear in status bar.
 */
BUBBAMON.setStatus = function(msg)
{
    document.getElementById("bubbaStatusText").value = msg;
}

/** @brief Update list of current downloads.
 *
 * This will update the list of current downloads.
 *
 * @param obj An array containing objects representing current downloads
 */
BUBBAMON.updateDownloads = function(obj)
{
    var total = 0;
    var finished = 0;
    var done;
    var bar;
    var list = document.getElementById("BM_SCR_downloads");
    // First we remove all current elements.
    while (list.firstChild) {
        list.removeChild(list.firstChild);
    }

    for(var key in obj.dls) {
        total++;
        var dl = obj.dls[key];

        var textField = {
            'name': dl.name.replace(/["'`]/g, ""),
            'status': "No info",
            'size': BUBBAMON.to_human(dl.size, 1),
            'down': '',
            'up': '',
            'seeds': '',
            'peers': '',
            'downloaded': '',
            'uploaded': '',
            'ratio': ''
        };

        if(dl.info.type == 'torrent') {
            textField['down'] = BUBBAMON.to_human(dl.info.download_rate, 1) + "/s";
            textField['up'] = BUBBAMON.to_human(dl.info.upload_rate, 1) + "/s";
            textField['seeds'] = dl.info.num_seeds;
            textField['peers'] = dl.info.num_peers;
            if(dl.downloaded != 0) {
                textField['ratio'] = (dl.info.total_upload * 1.0 / dl.downloaded).toFixed(2);
            } else {
                textField['ratio'] = '-'; 
            }
            textField['uploaded'] = BUBBAMON.to_human(dl.info.total_upload, 1);
            textField['downloaded'] = BUBBAMON.to_human(dl.downloaded, 1);
        } else {
            textField['down'] = BUBBAMON.to_human(dl.info.speed, 1) + "/s";
            textField['up'] = '-';
            textField['seeds'] = '-';
            textField['peers'] = '-';
            textField['ratio'] = '-';
            textField['uploaded'] = '.';
            textField['downloaded'] = BUBBAMON.to_human(dl.downloaded, 1);
        }

        try {
            switch(dl.status) {
            case 0:
            case 1:
                textField['status'] = BUBBAMON.locale.getText('bubbamon.status.queued');
                done = 0;
                break;
            case 2:
                if(dl.info.type == 'torrent') {
                    textField['status'] = BUBBAMON.locale.getText('bubbamon.status.seeding');
                } else {
                    textField['status'] = BUBBAMON.locale.getText('bubbamon.status.done');
                }
                // BUGFIX: Sometimes completed downloads report a download
                // speed. This will set it to 0B/s
                textField['down'] = "0B/s";
                done = 100;
                finished++;
                break;
            case 3:
                textField['status'] = BUBBAMON.locale.getText('bubbamon.status.canceling');
                done = -1;
                break;
            case 4:
                textField['status'] = BUBBAMON.locale.getText('bubbamon.status.canceled');
                done = -1;
                finished++;
                break;
            case 5:
                textField['status'] = BUBBAMON.locale.getText('bubbamon.status.downloading');
                done = dl.downloaded / dl.size * 100;
                break;
            case 6:
                textField['status'] = BUBBAMON.locale.getText('bubbamon.status.failed');
                done = -2;
                finished++;
                break;
            }
            var template = (done == 100) ? BUBBAMON.prefs.completed : BUBBAMON.prefs.ongoing;
    
            bar = new BUBBAMON.ProgressBar(
                BUBBAMON.parseTemplateText(template['left'], textField), 
                done, 
                BUBBAMON.parseTemplateText(template['upper'], textField), 
                BUBBAMON.parseTemplateText(template['lower'], textField));
        }
        catch(e) {
            bar = new BUBBAMON.ProgressBar(dl.name, done, BUBBAMON.locale.getText('bubbamon.status.unknown'), BUBBAMON.to_human(dl.size, 1));
        }

        var args = "'" + dl.uuid + "', '" + escape(dl.name) + "', '" + dl.source + "'";
        var download = bar.asXUL(textField);
        download.setAttribute("onclick", "BUBBAMON.promptForCancel(" + args + ");");
        list.appendChild(download);
    }

    if(total == 0) {
        var bar = new BUBBAMON.ProgressBar(BUBBAMON.locale.getText('bubbamon.dl.none'), -3, "", "");
        list.appendChild(bar.asXUL(null));
    }

    BUBBAMON.setStatus((total - finished) + ":" + finished);
}

BUBBAMON.parseTemplateText = function(format, data)
{
    var result =  format.replace("{size}", data['size']);
    result = result.replace("{downloaded}", data['downloaded']);
    result = result.replace("{uploaded}", data['uploaded']);
    result = result.replace("{status}", data['status']);
    result = result.replace("{seeds}", data['seeds']);
    result = result.replace("{peers}", data['peers']);
    result = result.replace("{down}", data['down']);
    result = result.replace("{up}", data['up']);
    result = result.replace("{ratio}", data['ratio']);
    result = result.replace("undefined", "-");
    result = result.replace("{name}", data['name']);

    return result;
}

/** @brief Update general information
 *
 * This will update the general information, like uptime and harddisk 
 * information.
 *
 * @param data Object containing general information
 */
BUBBAMON.updateGenerals = function(data)
{
    var deviceCol = document.getElementById("BM_VBOX_deviceColumn");
    var valueCol = document.getElementById("BM_VBOX_valueColumn");
    var partitionlist = document.getElementById("BM_VBX_partitioninfo");

    var uptime = data.uptime;
    var partitions = data.partitions;
    var disks = data.disks;
    var version = data.version;

    // Remove list of disks
    while(deviceCol.firstChild) {
        deviceCol.removeChild(deviceCol.firstChild);
        valueCol.removeChild(valueCol.firstChild);
    }
    // Empty current list of partitions
    while(partitionlist.firstChild) {
        partitionlist.removeChild(partitionlist.firstChild);
    }

    // Fill with new info
    for(var p in partitions) {
        var partition = partitions[p];
        var usage = 100 - (100 * partition.free / partition.size);
        var bar = new BUBBAMON.ProgressBar(p, usage, "", BUBBAMON.locale.getFormattedText('bubbamon.format.dl_size', [ BUBBAMON.to_human(partition.size, 1) ]));
        partitionlist.appendChild(bar.asXUL());
    }
    var found_temp =false;
    for(var d in disks) {
        var disk = disks[d];
        var info = document.createElement("label");
        info.setAttribute("value", d);
        deviceCol.appendChild(info);
        info = document.createElement("label");
        var temp = disk.temp;
        if(temp) {
            if(temp.match(/[0-9]+ C/)) {
                disk.temp = temp.replace(" ", "°");
            }
            found_temp = true;
        }
        info.setAttribute("value", disk.temp);
        valueCol.appendChild(info);
    }
    if(found_temp) {
        document.getElementById("BM_LBL_temperature").setAttribute("collapsed", "false");
        document.getElementById("BM_ROW_temperature").setAttribute("collapsed", "false");
    } else {
        document.getElementById("BM_LBL_temperature").setAttribute("collapsed", "true");
        document.getElementById("BM_ROW_temperature").setAttribute("collapsed", "true");
    }

    if(uptime[1] < 10)
        uptime[1] = "0" + uptime[1];
    if(uptime[2] < 10)
        uptime[2] = "0" + uptime[2];
    if(uptime[3] < 10)
        uptime[3] = "0" + uptime[3];

    // Next, do the uptime
    document.getElementById("BM_LBL_uptime").value = BUBBAMON.locale.getFormattedText('bubbamon.format.uptime', [ uptime[0], uptime[1], uptime[2], uptime[3] ]);

    document.getElementById("BM_LBL_version").value = version;

}

BUBBAMON.startUp = function() {
    BUBBAMON.locale = new BUBBAMON.StringBundle('BubbaMonStrings');
    BUBBAMON.prefs = new BUBBAMON.Preferences();

    // Make sure we present the correct version number
    try {
        var version;
        var oEM = Components.classes["@mozilla.org/extensions/manager;1"].
            getService(Components.interfaces.nsIExtensionManager);
        if (oEM.getItemForID) {
            version = oEM.getItemForID("bubbamon@thevoid.webhop.info").version;
        } else {
            version = oEM.getItemList("bubbamon@thevoid.webhop.info", null, {})[0].version;
        }
        document.getElementById("BM_LBL_title").value = "BubbaMon v" + version;
    }
    catch (e) {
        Components.utils.import("resource://gre/modules/AddonManager.jsm");
        AddonManager.getAddonByID("bubbamon@thevoid.webhop.info", function(addon) {
            document.getElementById("BM_LBL_title").value = "BubbaMon v" + addon.version;
        });
    }

    // Add a download item to context menu.
    var menu = document.getElementById("contentAreaContextMenu");
    menu.addEventListener("popupshowing", BUBBAMON.visibility, false);

    window.removeEventListener("load", BUBBAMON.startUp, true);

    // Do we know what kind of server we're using?
    if(BUBBAMON.prefs.configured)
    {
        if(BUBBAMON.prefs.serverType == 0) {
            BUBBAMON.detectServer();
        } 
        else
        {
            switch(BUBBAMON.prefs.serverType) {
                case 1:
                    BUBBAMON.server = new BUBBAMON.BubbaServer();
                    BUBBAMON.server.logOn();
                    break;
                case 2:
                    BUBBAMON.server = new BUBBAMON.BubbaTwo();
                    BUBBAMON.server.logOn();
                    break;
                default:
                    BUBBAMON.detectServer();
                    break;
            }
        }
    }
}

BUBBAMON.shutDown = function() {
    BUBBAMON.server.logOff();
    BUBBAMON.prefs.destroy();
}

BUBBAMON.detectServer = function() {
    if(BUBBAMON.detecting) {
        return;
    }
    BUBBAMON.detecting = true;
    var first_try = new XMLHttpRequest();
    first_try.open('GET', BUBBAMON.prefs.address + "/web-admin/index.php", true);
    first_try.onreadystatechange = function(aEvt) {
        if(first_try.readyState == 4) {
            if(first_try.status == 200) {
                // Most likely a BUBBA|server
                BUBBAMON.prefs.serverType = 1;
            } else {
                BUBBAMON.prefs.serverType = 0;
            }
            var second_try = new XMLHttpRequest();
            second_try.open('GET', BUBBAMON.prefs.address + "/admin/login/index", true);
            second_try.onreadystatechange = function(aEvt) {
                if(second_try.readyState == 4) {
                    if(second_try.status == 200) {
                        // Most likely a BUBBA|two
                        BUBBAMON.prefs.serverType |= 2;
                    } else {
                        BUBBAMON.prefs.serverType &= 253;
                    }

                    if((BUBBAMON.prefs.serverType != 1) &&
                            (BUBBAMON.prefs.serverType != 2)) {
                        var check = {value: false};
                        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                            .getService(Components.interfaces.nsIPromptService);

                        var button = prompts.confirmEx(null, BUBBAMON.locale.getText('bubbamon.detection.failed'), 
                                BUBBAMON.locale.getText('bubbamon.detection.gen1') + "\n" + BUBBAMON.locale.getText('bubbamon.detection.gen2'),
                                prompts.STD_YES_NO_BUTTONS, null, null, null, null, check);
                        if(button == 0) {
                            BUBBAMON.prefs.serverType = 1;
                        }
                        else {
                            BUBBAMON.prefs.serverType = 2;
                        }
                    }

                    switch(BUBBAMON.prefs.serverType) {
                        case 1:
                            BUBBAMON.server = new BUBBAMON.BubbaServer();
                            break;
                        case 2:
                            BUBBAMON.server = new BUBBAMON.BubbaTwo();
                            break;
                        default:
                            alert("Oops, this is weird. I've found an unknown type of bubba server.");
                            break;
                    }
                    BUBBAMON.prefs.prefs.setIntPref("serverType", BUBBAMON.prefs.serverType);

                    BUBBAMON.detecting = false;
                    BUBBAMON.server.logOn();
                }
            }
            second_try.send(null);
        }
    }
    first_try.send(null);
}

BUBBAMON.visibility = function(event) {
    if (gContextMenu.onImage || gContextMenu.onLink) {
        switch (BUBBAMON.prefs.downloadMode) {
            case 1:
                document.getElementById("bubbamon_addImmediate").hidden = false;
                document.getElementById("bubbamon_addIntermediate").hidden = true;
                document.getElementById("bubbamon_addDownload").hidden = true;
                break;
            case 2:
                document.getElementById("bubbamon_addImmediate").hidden = true;
                document.getElementById("bubbamon_addIntermediate").hidden = false;
                document.getElementById("bubbamon_addDownload").hidden = true;
                break;
            case 3:
                document.getElementById("bubbamon_addImmediate").hidden = true;
                document.getElementById("bubbamon_addIntermediate").hidden = true;
                document.getElementById("bubbamon_addDownload").hidden = false;
                break;
            default:
        }
    } else {
        document.getElementById("bubbamon_addImmediate").hidden = true;
        document.getElementById("bubbamon_addIntermediate").hidden = true;
        document.getElementById("bubbamon_addDownload").hidden = true;
    }
}

BUBBAMON.onMenu = function() {
    var link = new String(node);

    if(!BUBBAMON.prefs.configured) {
        alert(BUBBAMON.locale.getText("bubbamon.hint.not_configured"));
        return;
    }

    if(-1 != link.search(/(http|ftp):\/\//)) {
        BUBBAMON.addImmediateDownload(link);
    }
}

BUBBAMON.configure = function() {
	window.openDialog("chrome://bubbamon/content/options.xul", "BubbaMon Options", "centerscreen,chrome,modal");
}

BUBBAMON.addImmediateDownload = function(url) {
    // Not the best UUID generator, but it'll probably do for this job.
    var now = new Date();
    var t = now.getUTCMilliseconds();
    t += now.getUTCSeconds() * 1000;
    t += now.getUTCMinutes() * 60000;
    t += now.getUTCHours() * 3600000;
    t += now.getUTCDate() * 86400000;
    t += now.getUTCMonth() * 2073600000;
    t += now.getUTCFullYear() * 756864000000;

    var tuuid = "ftd" + t.toString(16);
    BUBBAMON.server.addDownload(encodeURIComponent(url), tuuid);
}

// This is only a validator that will test that it really is a torrent
// file that is being downloaded. No point in making an intermediate
// download on a regular file.
BUBBAMON.addIntermediateDownload = function(url) {
    var req = new XMLHttpRequest();
    req.open("HEAD", url, true);
    req.onreadystatechange = function()
    {
        if(req.readyState == 2)
        {
            var mime = req.getResponseHeader("Content-Type");
            if(mime == "application/x-bittorrent") {
                BUBBAMON.doIntermediateDownload(url);
            } else {
                BUBBAMON.addImmediateDownload(url);
            }
            req.open("HEAD", encodeURIComponent(url), true);
        }
    }
    req.send(null);
}


BUBBAMON.doIntermediateDownload = function(url) {
    var dir = BUBBAMON.getLocalFile(BUBBAMON.prefs.intermediateDst);
    if( !dir ) { return; }

    //Nasty way of getting a filename
    var temp = url.split("/");
    var file = temp[temp.length-1];
    if(file.length < 1) { file = temp[temp.length-2]; }
    if(!file.match(/\.torrent$/)) { file += ".torrent"; }
    file = unescape(file);
    dir.append(file);

    var output = Components.classes["@mozilla.org/network/file-output-stream;1"]
        .createInstance(Components.interfaces.nsIFileOutputStream);
    var buffer = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
        .createInstance(Components.interfaces.nsIBufferedOutputStream);
    output.init(dir, 0x02 | 0x08 | 0x20, 0664, null);
    buffer.init(output, 8192);

    var streamwriter = {
        onStartRequest: function(request, context) { },

        onDataAvailable: function onDataAvailable(request, context, stream, offset, count) {
            while(count > 0)
                count -= buffer.writeFrom(stream, count);
        },

        onStopRequest: function(request, context) {
            buffer.close();
            output.close();
            //add ending / in path if not already there
            if( !BUBBAMON.prefs.intermediateSrc.match(/\/$/) ) {
                BUBBAMON.prefs.intermediateSrc += "/";
            }
            BUBBAMON.addImmediateDownload(BUBBAMON.prefs.intermediateSrc+file);
        }
    };

    var nsIIOService = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);

    var nsIHttpChannel = nsIIOService.newChannel(url, "", null);
    nsIHttpChannel.QueryInterface(Components.interfaces.nsIHttpChannel);
    nsIHttpChannel.asyncOpen( streamwriter, dir );
}

BUBBAMON.promptForCancel = function(uuid, name, source) {
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
        .getService(Components.interfaces.nsIPromptService);
    var menu = document.getElementById("BM_POP_status");
    menu.hidePopup();
    var check = {
        value: false
    };
    var button = prompts.confirmEx(window, BUBBAMON.locale.getText("bubbamon.cancel.title"), 
            BUBBAMON.locale.getText("bubbamon.cancel.question") + "\n" + 
            BUBBAMON.locale.getFormattedText("bubbamon.cancel.file", [ unescape(name) ]), prompts.STD_YES_NO_BUTTONS, 
            null, null, null, null, check);
    if (button == 0) {
        BUBBAMON.server.cancelDownload(source, uuid);
    }

}

BUBBAMON.DragDropHandler = {
    getSupportedFlavours : function () {
        var flavours = new FlavourSet();
        flavours.appendFlavour("text/unicode");
        return flavours;
    },
    onDragStart: function (evt , transferData, action){},
    onDragOver: function (evt,flavour,session) {},
    onDrop : function (evt, transferData, session) {
        if(!BUBBAMON.prefs.configured) {
            alert(BUBBAMON.locale.getText("bubbamon.hint.not_configured"));
            return;
        }

        var td = transferData;
        try
        {
            switch(td.flavour.contentType)
            {
            case "text/unicode": {
                // Verify data begins with 'http' or 'ftp'
                if(-1 != td.data.search(/(http|ftp):\/\//)) {
                    switch (bubbamon_downloadMode) {
                    case 1:
                        BUBBAMON.addImmediateDownload(td.data);
                        break;
                    case 2:
                        BUBBAMON.addIntermediateDownload(td.data);
                        break;
                    case 3:
                        if(evt.shiftKey) {
                            BUBBAMON.addIntermediateDownload(td.data);
                        } else {
                            BUBBAMON.addImmediateDownload(td.data);
                        }
                        break;
                    default:
                        break;
                    }
                } else {
                    alert("Unknown type (" + td.data + ")");
                }
                }
                break;
            }
        }
        catch(ex) {
            alert(ex.name + "\n" + ex.message);
        }
    }
}

BUBBAMON.removeLocalTorrent = function(file) {
    var path = BUBBAMON.getLocalFile(BUBBAMON.prefs.intermediateDst);
    if( !path ) { return; }

    try {
        path.append(file);
        if(path.isFile()) {
            path.remove(false);
        }
    }
    catch(ex) { }
}

BUBBAMON.getLocalFile = function(file) {
    const nsILocalFile = Components.interfaces.nsILocalFile;

    var dir = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(nsILocalFile);

    try{
        dir.initWithPath(file);
        if(!dir.isDirectory()) {
            throw("error");
        }
    }
    catch(ex){
        alert("Bubbamon: \"Intermediate directory\" is not a valid directory.\n"+ex.name);
        return false;
    }

    return dir;
}

BUBBAMON.to_human = function(no, precision) {
    var txt = [ "B", "kB", "MB", "GB", "TB" ];
    var exp = 0;

    while((no/1024) > 1) {
        no = no / 1024;
        exp++;
    }
    var result = "0B";
    try {
        result = no.toFixed(precision) + txt[exp];
    }
    catch(e) {
        result = no;
    }
    return result;
}

